--[[
    Broker_RecZone by LordFarlander
    Version 3.0.70
    $Revision: 66 $

    A conversion of Fubar_RecZone by wmrojer
]]--

--[[
Copyright (c) 2008-2009, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Broker_RecZone" );
local qtip = LibStub( "LibQTip-1.0" );
local Jostle = LibStub( "LibJostle-3.0" );

Broker_RecZone = LibStub( "AceAddon-3.0" ):NewAddon( "Broker_RecZone", "AceEvent-3.0", "AceConsole-3.0", "AceTimer-3.0" );

Broker_RecZone.version = "3.0.70";
Broker_RecZone.date = ("$Date: 2009-05-27 22:44:34 +0000 (Wed, 27 May 2009) $"):sub( 8, 17 );

local clickProvider, clickPrototype, baseclickCellPrototype = qtip:CreateCellProvider( qtip.LabelProvider );

function clickPrototype:InitializeCell()
    baseclickCellPrototype.InitializeCell( self );
end--clickPrototype:InitializeCell()

function clickPrototype:SetupCell( tooltip, value, justification, font, class, zone )
    self:EnableMouse( true );
    self:SetScript( "OnEnter", function( self )
                                   if( not self.highlight ) then
                                       self.highlight = self:CreateTexture( nil, "BACKGROUND" );
                                       self.highlight:Hide();
                                   end--if
                                   self.highlight:SetTexture( "Interface\\QuestFrame\\UI-QuestTitleHighlight" );
                                   self.highlight:SetBlendMode( "ADD" );
                                   self.highlight:SetAllPoints( self );
                                   self.highlight:Show();
                               end );
    self:SetScript( "OnLeave", function( self )
                                   self.highlight:ClearAllPoints();
                                   self.highlight:Hide();
                               end );
    self:SetScript( "OnMouseDown", function() class:OnClickZone( zone ) end );
    return baseclickCellPrototype.SetupCell( self, tooltip, value, justification, font );
end--clickPrototype:SetupCell( tooltip, value, justification, font, class, zone )

-- Enumerations.
-- Faction
BRECZONE_ALLIANCE      = 1;
BRECZONE_HORDE         = 2;
BRECZONE_CONTESTED     = 128;
BRECZONE_SANCTUARY     = 255;
-- Continent
BRECZONE_KALIMDOR      = 1;
BRECZONE_EASTERN       = 2;
BRECZONE_OUTLAND       = 3;
BRECZONE_NORTHREND     = 4;
-- take into account future continents, move "special" ids to big numbers
BRECZONE_NOWHERE       = 128;
BRECZONE_OTHER         = 255;
-- Type
BRECZONE_ZONE          = 1;
BRECZONE_CITY          = 2;
BRECZONE_TRANSPORT     = 3;
BRECZONE_NONE          = 4;
BRECZONE_PVP           = 5;
BRECZONE_COLLECTOR     = 6;
BRECZONE_DUNGEON       = 7;
BRECZONE_RAID          = 8;
BRECZONE_BATTLEGROUND  = 9;
BRECZONE_ARENA         = 10;
BRECZONE_UNKNOWN       = 255;

Broker_RecZone.vars = {
    current_continent = nil,
    current_zone = nil,
    playerLevel = 0,
    zoneName = "",
    subZoneName = "",
    playerFaction = 0,
    zonetext = "",
    coordtext = "",
    leveltext = "",
    playerCoordX = 0,
    playerCoordY = 0,
    factions = {
        [BRECZONE_ALLIANCE]     = FACTION_ALLIANCE,
        [BRECZONE_HORDE]        = FACTION_HORDE,
        [BRECZONE_CONTESTED]    = L["Contested"],
        [BRECZONE_SANCTUARY]    = L["Sanctuary"],
        [BRECZONE_PVP]          = L["PVP"],
    },
    types = {
        [BRECZONE_ZONE]         = "",
        [BRECZONE_CITY]         = L["City"],
        [BRECZONE_TRANSPORT]    = L["Transport"],
        [BRECZONE_NONE]         = "",
        [BRECZONE_DUNGEON]      = "",
        [BRECZONE_BATTLEGROUND] = L[" (Battleground)"],
        [BRECZONE_ARENA]        = L[" (Arena)"],
        [BRECZONE_PVP]          = L["PVP"],
        [BRECZONE_RAID]         = "",
        [BRECZONE_COLLECTOR]    = "",
        [BRECZONE_UNKNOWN]      = "",
    },
    continents = {},
};

-- Font Colors
local HEX_WHITE = "|cffffffff";
local HEX_RED = "|cffff0000";
local HEX_ORANGE = "|cffff7700";
local HEX_YELLOW = "|cffffff00";
local HEX_GREEN = "|cff00ff00";
local HEX_GRAY = "|cff777777";
local HEX_LIGHTBLUE = "|cff77ccff";

function Broker_RecZone:GetZoneColor( zone )
    -- PlayerLevel <= low for zone - 4  == RED
    -- PlayerLevel <  low for zone      == ORANGE
    -- PlayerLevel <= high              == YELLOW
    -- PlayerLevel <  high + 5          == GREEN
    -- Otherwise                        == GRAY
    local type = zone.type;
    local low = zone.low;
    local high = zone.high;
    local vars = self.vars;
    local playerLevel = vars.playerLevel;

    if( ( type == BRECZONE_CITY ) and ( zone.faction == vars.playerFaction ) ) then
        return HEX_WHITE;
    elseif( type == BRECZONE_SANCTUARY ) then
        return HEX_LIGHTBLUE;
    elseif( ( type == BRECZONE_PVP ) or ( playerLevel <= ( low - 4 ) ) ) then
        return HEX_RED;
    elseif( playerLevel < low ) then
        return HEX_ORANGE;
    elseif( playerLevel <= high ) then
        return HEX_YELLOW;
    elseif( playerLevel < ( high + 5 ) ) then
        return HEX_GREEN;
    else
        return HEX_GRAY;
    end--if
end--Broker_RecZone:GetZoneColor( zone )

function Broker_RecZone:GetInstanceText( instance, colors, difficulty )
    local color = HEX_WHITE;
    local text = "";
    local low = instance.low;
    local high = instance.high;

    if( not difficulty ) then
        difficulty = 1;
    end--if
    if( instance == nil ) then
        return "";
    end--if
    if( ( difficulty == 2 ) and instance.heroic ) then
        low = instance.heroic.low;
        high = instance.heroic.high;
    elseif( ( difficulty == 3 ) and instance.epic ) then
        low = instance.epic.low;
        high = instance.epic.high;
    end--if
    if( colors ) then
        color = self:GetZoneColor( instance );
    end--if
    if( instance.type == BRECZONE_RAID ) then
        if( instance.heroic and instance.heroic.size ) then
            text = instance.name .. L[" (Raid %i or %i)"]:format( instance.size, instance.heroic.size );
        else
            text = instance.name .. L[" (Raid %s)"]:format( instance.size or "?" );
        end--if
    elseif( instance.type ~= BRECZONE_DUNGEON ) then
        text = ( instance.displayname or instance.name ) .. self.vars.types[instance.type];
    elseif( instance.low == instance.high ) then
        text = instance.name .. self.vars.types[instance.type] .. (" (%i+)"):format( low );
    else
        text = instance.name .. L[" (%i+ to %i+)"]:format( low, high );
    end--if
    if( ( difficulty > 1 ) and self.db.profile.showIncreasedDifficulty ) then
        return ("%s%s (%s)|r"):format( color, text, _G["DUNGEON_DIFFICULTY" .. difficulty] );
    end--if
    return color .. text .. "|r";
end--Broker_RecZone:GetInstanceText( instance, colors )

function Broker_RecZone:GetZoneText( zone, colors )
    local color = HEX_WHITE;
    local text = zone.name;

    if( zone == nil ) then
        return "";
    end
    if( colors ) then
        color = self:GetZoneColor( zone );
    end

    local faction = zone.faction;
    local type = zone.type;

    -- No level range for friendly city zones
    if( ( ( type == BRECZONE_CITY ) or ( type == BRECZONE_TRANSPORT ) ) and ( faction == self.vars.playerFaction ) ) then
        text = zone.name;
        color = HEX_WHITE;
    elseif( faction == BRECZONE_SANCTUARY ) then
        text = zone.name;
        color = HEX_LIGHTBLUE;
    elseif( ( type == BRECZONE_DUNGEON ) or ( type == BRECZONE_COLLECTOR ) ) then
        if( zone.low == zone.high ) then
            text = zone.displayname or zone.name .. (" (%i+)"):format( zone.low ); --  (60+)
        else
            text = zone.displayname or zone.name .. L[" (%i+ to %i+)"]:format( zone.low, zone.high ); --  (50+ - 58+)
        end--if
    elseif( ( type == BRECZONE_BATTLEGROUND  ) or ( type == BRECZONE_PVP ) or ( type == BRECZONE_ARENA ) ) then
        text = zone.name .. self.vars.types[zone.type];
    elseif( type == BRECZONE_RAID ) then
        if( zone.heroic and zone.heroic.size ) then
            text = zone.name .. L[" (Raid %i or %i)"]:format( zone.size, zone.heroic.size );
        else
            text = zone.name .. L[" (Raid %s)"]:format( zone.size or "?" );
        end--if
    elseif( type ~= BRECZONE_UNKNOWN ) then
        text = zone.name .. L[" (%i to %i)"]:format( zone.low, zone.high ); -- (1 - 12 )
    end--if
    -- Add zone name and level range.
    return color .. text .. "|r";
end--Broker_RecZone:GetZoneText( zone, colors )

function Broker_RecZone:GetFactionColor( zone, colors )
    if( not zone ) then
        return "";
    end--if
    if( colors ) then
        local faction = zone.faction;

        if( faction == BRECZONE_CONTESTED ) then
            return HEX_YELLOW;
        elseif( faction == BRECZONE_SANCTUARY ) then
            return HEX_LIGHTBLUE;
        elseif( faction == self.vars.playerFaction ) then
            return HEX_GREEN;
        else
            return HEX_RED;
        end--if
    end--if
    return "";
end--Broker_RecZone:GetFactionColor( zone, colors )

function Broker_RecZone:GetFactionText( zone, colors )
    local color = HEX_WHITE;

    if( zone == nil ) then
        return "";
    end
    if( colors ) then
        color = self:GetFactionColor( zone, colors );
    end
    return color .. self.vars.factions[zone.faction] .. "|r";
end--Broker_RecZone:GetFactionText( zone, colors )

function Broker_RecZone:GetContinentText( zone, colors )
    local color = HEX_WHITE;

    if( zone == nil ) then
        return "";
    end--if
    if( colors ) then
        if( zone.continent == self.vars.current_continent ) then
            color = HEX_GREEN;
        else
            color = HEX_YELLOW;
        end--if
    end--if
    return color .. self.vars.continents[zone.continent] .. "|r";
end--Broker_RecZone:GetContinentText( zone, colors )

function Broker_RecZone:GetLevelText( zone, colors, difficulty )
    local color = HEX_WHITE;
    local low = zone.low;
    local high = zone.high;
    
    if( not difficulty ) then
        difficulty = 1;
    end--if
    if( ( difficulty == 2 ) and zone.heroic ) then
        low = zone.heroic.low;
        high = zone.heroic.high;
    elseif( ( difficulty == 3 ) and zone.epic ) then
        low = zone.epic.low;
        high = zone.epic.high;
    end--if
    if ( colors ) then
        color = self:GetZoneColor( zone );
    end
    if( ( zone.type == BRECZONE_CITY ) and ( ( zone.faction == self.vars.playerFaction ) or ( zone.faction == BRECZONE_SANCTUARY ) ) ) then
        return HEX_WHITE .. self.vars.types[zone.type] .. "|r";
    elseif (zone.faction == BRECZONE_SANCTUARY) then
        return HEX_LIGHTBLUE .. self.vars.types[zone.type] .. "|r";
    elseif( ( zone.type == BRECZONE_DUNGEON ) or ( zone.type == BRECZONE_COLLECTOR ) ) then
        if( zone.low == zone.high ) then
            return color .. (" %i+|r"):format( low ); -- 60+
        else
            return color .. (" %i+-%i+|r"):format( low , high ); --  50+-58+
        end--if
    elseif( ( zone.type == BRECZONE_BATTLEGROUND ) or ( zone.type == BRECZONE_PVP ) or ( zone.type == BRECZONE_ARENA ) ) then
        return color .. self.vars.types[zone.type] .. "|r";
    elseif( zone.type == BRECZONE_RAID ) then
        if( zone.heroic and zone.heroic.size ) then
            text = color .. L[" (Raid %i or %i)"]:format( zone.size, zone.heroic.size ) .. "|r";
        else
            text = color .. L[" (Raid %s)"]:format( zone.size or "?" ) .. "|r";
        end--if
    elseif( zone.type == BRECZONE_UNKNOWN ) then
        return "";
    end--if
    if( low == high ) then
        return color .. tostring( low ) .. "|r";
    end--if
    return color .. (" %i-%i|r"):format( low , high );
end--Broker_RecZone:GetLevelText( zone, colors )

function Broker_RecZone:GetPathText( zone, colors )
    local zonecolor = HEX_WHITE;
    local levelcolor = HEX_WHITE;

    if ( colors ) then
        zonecolor = self:GetFactionColor( zone, colors );
        levelcolor = self:GetZoneColor( zone );
    end--if
    if( ( zone.type == BRECZONE_CITY ) or ( zone.type == BRECZONE_TRANSPORT ) ) then
        return zonecolor .. zone.name .. "|r";
    elseif( ( zone.type == BRECZONE_DUNGEON ) or ( zone.type == BRECZONE_COLLECTOR ) ) then
        if( zone.low == zone.high ) then
            return zonecolor .. ( zone.displayname or zone.name ) .. "|r " .. levelcolor .. ("(%i+)|r"):format( zone.low );
        else
            return zonecolor .. ( zone.displayname or zone.name ) .. "|r " .. levelcolor .. ("(%i+-%i+)|r"):format( zone.low, zone.high );
        end--if
    elseif( ( zone.type == BRECZONE_BATTLEGROUND ) or ( zone.type == BRECZONE_PVP ) or ( zone.type == BRECZONE_ARENA ) ) then
        return zonecolor .. zone.name .. "|r" .. levelcolor .. self.vars.types[zone.type] .. "|r";
    elseif( zone.type == BRECZONE_RAID ) then
        if( zone.heroic and zone.heroic.size ) then
            text = zonecolor .. L[" (Raid %i or %i)"]:format( zone.size, zone.heroic.size ) .. "|r";
        else
            text = zonecolor .. L[" (Raid %s)"]:format( zone.size or "?" ) .. "|r";
        end--if
    end--if
    if( zone.low == zone.high ) then
        return zonecolor .. zone.name .. "|r " .. levelcolor .. ("(%i)|r"):format( zone.low );
    end--if
    return zonecolor .. zone.name .. "|r " .. levelcolor .. ("(%i-%i)|r"):format( zone.low, zone.high );
end--Broker_RecZone:GetPathText( zone, colors )

function Broker_RecZone:StartCoordsUpdate()
    local x, y = GetPlayerMapPosition( "player" );

    if( x == 0 and y == 0 ) then -- instance
        self.timerHandle = self:ScheduleRepeatingTimer( self.OnCoordUpdate, 60, self );
        self.vars.playerCoordX = 0;
        self.vars.playerCoordY = 0;
        self.vars.coordtext = "";
    else
        self.timerHandle = self:ScheduleRepeatingTimer( self.OnCoordUpdate, 0.1, self );
    end--if
end--Broker_RecZone:StartCoordsUpdate()

function Broker_RecZone:StopCoordsUpdate()
    if( self.timerHandle ) then
        self:CancelTimer( self.timerHandle );
        self.timerHandle = nil;
    end--if
    self.vars.playerCoordX = 0;
    self.vars.playerCoordY = 0;
    self.vars.coordtext = "";
end--Broker_RecZone:StopCoordsUpdate()

Broker_RecZone.vars.playerinworld = false;

function Broker_RecZone:OnInitialize()
    local AceConfigReg = LibStub( "AceConfigRegistry-3.0" );
    local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
    local defaults = {
        profile = {
            showCurInstance = true,
            showInstance = true,
            showBattleground = true,
            showRaid = false,
            showFaction = false,
            showContinent = false,
            showLoc = true,
            showMap = true,
            lower = 5,
            higher = 5,
            showColoredText = true,
            showZoneName = true,
            showSubZoneName = false,
            showCoords = true,
            showMapCoords = true,
            minimapBar = false,
            showIncreasedDifficulty = true;
            showLevelRange = true;
        },
    };
    local options = {
        type = "group",
        handler = self,
        args = {
            label = {
                type = "group",
                name = L["Label"],
                desc = L["Label"],
                order = 1,
                args = {
                    zone = {
                        type = "toggle",
                        name = L["Show zone name"],
                        desc = L["Show zone name"],
                        get = function( info )
                                  return info.handler.db.profile.showZoneName;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showZoneName = value;
                                  info.handler:Update();
                              end,
                        order = 1,
                    },
                    subzone = {
                        type = "toggle",
                        name = L["Show sub zone name"],
                        desc = L["Show sub zone name"],
                        get = function( info )
                                  return info.handler.db.profile.showSubZoneName;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showSubZoneName = value;
                                  info.handler:Update();
                              end,
                        order = 2,
                    },
                    levelrange = {
                        type = "toggle",
                        name = L["Show level range"],
                        desc = L["Show level range"],
                        get = function( info )
                                  return info.handler.db.profile.showLevelRange;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showLevelRange = value;
                                  info.handler:Update();
                              end,
                        order = 2,
                    },
                    coords = {
                        type = "toggle",
                        name = L["Show coords"],
                        desc = L["Show coords"],
                        get = function( info )
                                  return info.handler.db.profile.showCoords;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showCoords = value;
                                  if( value ) then
                                      info.handler:StartCoordsUpdate();
                                  else
                                      info.handler:StopCoordsUpdate();
                                  end--if
                                  info.handler:Update();
                              end,
                        order = 3,
                    },
                },
            },
            tooltip = {
                type = "group",
                name = L["Tooltip"],
                desc = L["Tooltip"],
                order = 2,
                args = {
                    curinstance = {
                        type = "toggle",
                        name = L["Show instances for current zone"],
                        desc = L["Show instances for current zone"],
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.showCurInstance;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showCurInstance = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 1,
                    },
                    faction = {
                        type = "toggle",
                        name = L["Show faction of zones"],
                        desc = L["Show faction of zones"],
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.showFaction;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showFaction = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 2,
                    },
                    continent = {
                        type = "toggle",
                        name = L["Show continent for zones"],
                        desc = L["Show continent for zones"],
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.showContinent;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showContinent = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 3,
                    },
                    instance = {
                        type = "toggle",
                        name = L["Show recommended instances"],
                        desc = L["Show recommended instances"],
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.showInstance;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showInstance = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 4,
                    },
                    battleground = {
                        type = "toggle",
                        name = L["Show battlegrounds"],
                        desc = L["Show battlegrounds"],
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.showBattleground;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showBattleground = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 5,
                    },
                    raid = {
                        type = "toggle",
                        name = L["Show raid instances"],
                        desc = L["Show raid instances"],
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.showRaid;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showRaid = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 6,
                    },
                    loc = {
                        type = "toggle",
                        name = L["Show location of instances"],
                        desc = L["Show location of instances"],
                        width = "double",
                        get = function( info )
                                  return info.handler.db.profile.showLoc;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showLoc = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 7,
                    },
                    lower = {
                        type = "range",
                        name = L["Recommend lower level zones"],
                        desc = L["Recommend lower level zones"],
                        width = "double",
                        max = 10;
                        min = 0;
                        step = 1;
                        get = function( info )
                                  return info.handler.db.profile.lower;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.lower = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 8,
                    },
                    higher = {
                        type = "range",
                        name = L["Recommend higher level zones"],
                        desc = L["Recommend higher level zones"],
                        width = "double",
                        max = 10;
                        min = 0;
                        step = 1;
                        get = function( info )
                                  return info.handler.db.profile.higher;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.higher = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 9,
                    },
                },
            },
            map = {
                type = "group",
                name = L["Map"],
                desc = L["Map"],
                order = 3,
                args = {
                    mapoverlay = {
                        type = "toggle",
                        name = L["Show zone levels"],
                        desc = L["Show zone levels"],
                        get = function( info )
                                  return info.handler.db.profile.showMap;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showMap = value;
                                  if( value ) then
                                      Broker_RecZoneMapText:Show();
                                  else
                                      Broker_RecZoneMapText:Hide();
                                  end
                              end,
                        order = 1,
                    },
                    mapcoords = {
                        type = "toggle",
                        name = L["Show coords"],
                        desc = L["Show coords"],
                        get = function( info )
                                  return info.handler.db.profile.showMapCoords;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showMapCoords = value;
                                  if( value ) then
                                      Broker_RecZoneMapFrameCursorCoords:Show();
                                      Broker_RecZoneMapFramePlayerCoords:Show();
                                  else
                                      Broker_RecZoneMapFrameCursorCoords:Hide();
                                      Broker_RecZoneMapFramePlayerCoords:Hide();
                                  end--if
                              end,
                        order = 2,
                    },
                },
            },
            minimapBar = {
                type = "toggle",
                name = L["Show minimap bar"],
                desc = L["Show the bar above the minimap that tells the location and allows you to close minimap."],
                get = function( info )
                          return info.handler.db.profile.minimapBar
                      end,
                set = function( info, value )
                          info.handler.db.profile.minimapBar = value;
                          if( value ) then
                              MinimapBorderTop:Show();
                              MinimapToggleButton:Show();
                              MinimapZoneTextButton:Show();
                          else
                              MinimapBorderTop:Hide();
                              MinimapToggleButton:Hide();
                              MinimapZoneTextButton:Hide();
                          end--if
                          Jostle:Refresh();
                      end,
            },
            color = {
                type = "toggle",
                name = L["Show colored text"],
                desc = L["Show colored text"],
                get = function( info )
                          return info.handler.db.profile.showColoredText;
                      end,
                set = function( info, value )
                          info.handler.db.profile.showColoredText = value;
                          info.handler:Update();
                      end,
                order = 91,
            },
            showIncreasedDifficulty = {
                type = "toggle",
                name = L["Show increaded difficulty"],
                desc = L["Put the difficulty of the dungeon after the name if it is not normal difficulty."],
                width = "double",
                get = function( info )
                          return info.handler.db.profile.showColoredText;
                      end,
                set = function( info, value )
                          info.handler.db.profile.showColoredText = value;
                          info.handler:Update();
                      end,
                order = 92,
            },
        },
    };

    if( RecZone ) then
        options.args.copysettings = {
            type = "execute",
            order = -1,
            name = L["Copy setting from FuBar_RecZone"],
            desc = L["Copy setting from FuBar_RecZone"],
            width = "double",
            func = function( info )
                       local function DeleteAll( table )
                           for i, _ in pairs( table ) do
                               table[i] = nil;
                           end--if
                       end--function DeleteAll( table )
                       local function CopyTable( to, from )
                           for i, v in pairs( from ) do
                               if( type( v ) == "table" ) then
                                   to[i] = {};
                                   CopyTable( to[i], v );
                               else
                                   to[i] = v;
                               end--if
                           end--for
                       end--CopyTable( to, from )

                       DeleteAll( self.db.profile );
                       CopyTable( self.db.profile, RecZone.db.profile );
                       info.handler.db.profile = RecZone.db.profile;
                   end,
        };
    end--if

    self.db = LibStub( "AceDB-3.0" ):New( "Broker_RecZoneDB", defaults );
    
    local profileOptions = LibStub( "AceDBOptions-3.0" ):GetOptionsTable( self.db );
    self.db.RegisterCallback( self, "OnProfileChanged" );
    self.db.RegisterCallback( self, "OnProfileReset" );

    AceConfigReg:RegisterOptionsTable( "Broker_RecZone Main Options", options );
    AceConfigReg:RegisterOptionsTable( "Broker_RecZone Profile", profileOptions );
    AceConfigDialog:AddToBlizOptions( "Broker_RecZone Main Options", L["Recommended Zone"] );
    AceConfigDialog:AddToBlizOptions( "Broker_RecZone Profile", L["Profile"], L["Recommended Zone"] );

    self.ldbobj = LibStub( "LibDataBroker-1.1" ):NewDataObject( L["Recommended Zone"], {
            type = "data source",
            icon = "Interface\\AddOns\\Broker_RecZone\\Icon",
            label = L["Recommended Zone"],
            text = L["Recommended Zone"], -- To get Button Bin to work correctly when autohiding text
            OnClick = function( _, button )
                          self:OnClick( button );
                      end,
            OnEnter = function( frame )
                          if( frame and ( not InCombatLockdown() ) ) then
                              local tooltip = qtip:Acquire( "Broker_RecZone_Tooltip", 3, "LEFT", "CENTER", "RIGHT" );
    
                              self.vars.qtip = tooltip;
                              tooltip:EnableMouse( true );
                              self:UpdateTooltip();
                              tooltip:SmartAnchorTo( frame );
                              tooltip:Show();
                          end--if
                      end,
            OnLeave = function( frame )
                          local function HideTooltip()
                              if( MouseIsOver( self.vars.qtip ) ) then
                                  return;
                              end;
                              self.vars.qtip:SetScript( "OnLeave", nil );
                              qtip:Release( self.vars.qtip );
                              self.vars.qtip = nil;
                          end--HideTooltip()

                          if( self.vars.qtip ) then
                              if( MouseIsOver( self.vars.qtip ) ) then
                                  self.vars.qtip:SetScript( "OnLeave", HideTooltip );
                              else
                                  HideTooltip();
                              end--if
                          end--if
                      end,
        } );

    self.vars.continents = { GetMapContinents() };
    self.vars.continents[BRECZONE_NOWHERE] = "";
    self.vars.continents[BRECZONE_OTHER] = L["Unknown"];

    local mapFrame = CreateFrame( "Frame", "Broker_RecZoneMapFrame", WorldMapFrame );

    mapFrame:SetScript( "OnUpdate", self.MapFrame_OnUpdate );
    
    local text = mapFrame:CreateFontString( "Broker_RecZoneMapText", "OVERLAY" );
    local font, size = GameFontNormal:GetFont();

    text:SetFont(font, size, "OUTLINE")
    text:SetTextColor( GameFontNormal:GetTextColor() );
    text:SetShadowColor( GameFontNormal:GetShadowColor() );
    text:SetShadowOffset( GameFontNormal:GetShadowOffset() );
    text:SetPoint( "TOP", WorldMapFrameAreaLabel, "BOTTOM", 0, -5 );

    local cursorCoords = mapFrame:CreateFontString( "Broker_RecZoneMapFrameCursorCoords", "OVERLAY" );

    cursorCoords:SetFont( GameFontNormal:GetFont() );
    cursorCoords:SetTextColor( GameFontNormal:GetTextColor() );
    cursorCoords:SetShadowColor( GameFontNormal:GetShadowColor() );
    cursorCoords:SetShadowOffset( GameFontNormal:GetShadowOffset() );
    cursorCoords:SetPoint( "RIGHT", WorldMapFrame, "CENTER", -80, -367 );
    
    local playerCoords = mapFrame:CreateFontString( "Broker_RecZoneMapFramePlayerCoords", "OVERLAY" );

    playerCoords:SetFont( GameFontNormal:GetFont() );
    playerCoords:SetTextColor( GameFontNormal:GetTextColor() );
    playerCoords:SetShadowColor( GameFontNormal:GetShadowColor() );
    playerCoords:SetShadowOffset( GameFontNormal:GetShadowOffset() );
    playerCoords:SetPoint( "RIGHT", WorldMapFrame, "CENTER", 80, -367 );
end--Broker_RecZone:OnInitialize()

function Broker_RecZone:OnEnable()
    self:RegisterEvent( "MINIMAP_ZONE_CHANGED", "Update" );
    self:RegisterEvent( "PLAYER_ENTERING_WORLD", "OnEnteringWorld" );
    self:RegisterEvent( "PLAYER_LEVEL_UP", "Update" );
    self:RegisterEvent( "ZONE_CHANGED", "Update" );
    self:RegisterEvent( "ZONE_CHANGED_INDOORS", "Update" );
    self:RegisterEvent( "ZONE_CHANGED_NEW_AREA", "Update" );
    if( Broker_RecZoneMapFrame ) then
        Broker_RecZoneMapFrame:Show();
    end--if
    self.vars.playerinworld = true; -- Maybe not completly true, but good enough
    self:OnProfileChanged();
end--Broker_RecZone:OnEnable()

function Broker_RecZone:OnDisable()
    Broker_RecZoneMapFrame:Hide();
    Broker_RecZoneMapFrameCursorCoords:Hide();
    Broker_RecZoneMapFramePlayerCoords:Hide();
    if( self.db.profile.minimapBar ) then
        MinimapBorderTop:Show();
        MinimapToggleButton:Show();
        MinimapZoneTextButton:Show();
    end--if
end--Broker_RecZone:OnDisable()

function Broker_RecZone:OnProfileReset()
    self:OnProfileChanged();
end--Broker_RecZone:OnProfileReset()

function Broker_RecZone:OnProfileChanged()
    if( self.vars.playerinworld ) then
        if( self.db.profile.showCoords ) then
            self:StartCoordsUpdate();
        else
            self:StopCoordsUpdate();
        end--if
        if( self.db.profile.showMap ) then
            Broker_RecZoneMapText:Show();
        else
            Broker_RecZoneMapText:Hide();
        end
        if( self.db.profile.showMapCoords ) then
            Broker_RecZoneMapFrameCursorCoords:Show();
            Broker_RecZoneMapFramePlayerCoords:Show();
        else
            Broker_RecZoneMapFrameCursorCoords:Hide();
            Broker_RecZoneMapFramePlayerCoords:Hide();
        end--if
        if( self.db.profile.minimapBar ) then
            MinimapBorderTop:Show();
            MinimapToggleButton:Show();
            MinimapZoneTextButton:Show();
        else
            MinimapBorderTop:Hide();
            MinimapToggleButton:Hide();
            MinimapZoneTextButton:Hide();
        end--if
        Jostle:Refresh();
        self:Update();
    end--if
end--Broker_RecZone:OnProfileChanged()

function Broker_RecZone:OnEnteringWorld()
    if( not self.vars.playerinworld ) then
        self.vars.playerinworld = true;
    end--if
    if( self.db.profile.showCoords ) then
        self:StartCoordsUpdate();
    end--if
    self:Update();
end--Broker_RecZone:OnEnteringWorld()

function Broker_RecZone:OnCoordUpdate()
    local x, y = GetPlayerMapPosition( "player" );
    local vars = self.vars;

    if( self.db.profile.showCoords and ( x ~= 0 ) and ( y ~= 0 ) ) then
        if( ( vars.playerCoordX ~= x ) or ( vars.playerCoordY ~= y ) ) then
            vars.coordtext = ("(%.0f, %.0f) "):format( x * 100, y * 100 );
            vars.playerCoordX = x;
            vars.playerCoordY = y;
            self:OnTextUpdate();
        end--if
    else
        vars.playerCoordX = x;
        vars.playerCoordY = y;
        vars.coordtext = "";
        self:OnTextUpdate();
    end--if
end--Broker_RecZone:OnCoordUpdate()

function Broker_RecZone:OnDataUpdate()
    local i;
    local vars = self.vars;
    local profile = self.db.profile;

    vars.playerLevel = UnitLevel( "player" );
    vars.zoneName = GetRealZoneText();
    vars.subZoneName = GetSubZoneText();
    if( select( 2, UnitFactionGroup( "player" ) ) == FACTION_ALLIANCE ) then
        vars.playerFaction = BRECZONE_ALLIANCE;
    else
        vars.playerFaction = BRECZONE_HORDE;
    end--if

    vars.zonetext = "";
    if( profile.showZoneName ) then
        vars.zonetext = self.vars.zoneName .. " ";
        if( profile.showSubZoneName ) then
            if( vars.subZoneName ~= "" ) then
                vars.zonetext = vars.zoneName .. ": " .. vars.subZoneName .. " ";
            end--if
        end--if
    elseif( profile.showSubZoneName ) then
        if( vars.subZoneName ~= "" ) then
            vars.zonetext = vars.subZoneName .. " ";
        else
            vars.zonetext = vars.zoneName .. " ";
        end--if
    end--if

    vars.leveltext = "";

    local found = false;

    for _, zone in ipairs( vars.zones ) do
        if( string.find( vars.zoneName, zone.name, 1, true ) ) then
            local difficulty = GetInstanceDifficulty();

            vars.current_continent = zone.continent;
            vars.current_zone = zone;
            vars.leveltext = self:GetLevelText( zone, profile.showColoredText, difficulty );
            if( ( difficulty > 1 ) and profile.showIncreasedDifficulty ) then
                vars.leveltext = ("%s (%s)"):format( vars.leveltext, _G["DUNGEON_DIFFICULTY" .. difficulty] );
            end--if
            found = true;
            break;
        end--if
    end--for

    if( not found ) then
        local continent = BRECZONE_OTHER;
        local tempzone = { name = vars.zoneName, low = 0, high = 0, faction = BRECZONE_CONTESTED, type = BRECZONE_UNKNOWN, continent = BRECZONE_OTHER, };
        local _, instanceType = IsInInstance();
        local pvpinfo = GetZonePVPInfo();
        local difficulty = GetInstanceDifficulty();

        if( pvpinfo == "friendly" ) then
            tempzone.faction = vars.playerFaction;
        elseif( pvpinfo == "contested" ) then
            tempzone.faction = BRECZONE_CONTESTED;
            tempzone.type = BRECZONE_ZONE;
        elseif( pvpinfo == "hostile" ) then
            if( vars.playerFaction == FACTION_ALLIANCE) then
                tempzone.faction = FACTION_HORDE;
            else
                tempzone.faction = FACTION_ALLIANCE;
            end--if
        elseif( pvpinfo == "sanctuary" ) then
            tempzone.faction = BRECZONE_SANCTUARY;
            tempzone.type = BRECZONE_CITY;
        elseif( pvpinfo == "combat" ) then
            tempzone.faction = BRECZONE_PVP;
            tempzone.type = BRECZONE_PVP;
        end--if
        if( instanceType == "pvp" ) then
            tempzone.type = BRECZONE_BATTLEGROUND;
        elseif( instanceType == "arena" ) then
            tempzone.type = BRECZONE_ARENA;
        elseif( instanceType == "party" ) then
            tempzone.type = BRECZONE_DUNGEON;
        elseif( instanceType == "raid" ) then
            tempzone.type = BRECZONE_RAID;
        end--if
        if( vars.zoneName and ( vars.zoneName ~= "" ) ) then
            local i = 1;

            self:Print( L["Unknown zone detected \"%s\""], vars.zoneName )
            table.insert( vars.zones, tempzone );

            --attempt to determine continent
            while true do
                if( not vars.continents[i] ) then
                    break;
                end--if
                for _, name in pairs( { GetMapZones( i ) } ) do
                    if( name == vars.zoneName ) then
                        continent = i;
                        break;
                    end--if
                end--for
                i = i + 1;
            end--while
            tempzone.continent = continent;
            table.insert( vars.zones, tempzone );            
        end--if
        vars.current_continent = tempzone.continent;
        vars.current_zone = tempzone;
        vars.leveltext = self:GetLevelText( tempzone, profile.showColoredText, difficulty );
        if( ( difficulty > 1 ) and profile.showIncreasedDifficulty ) then
            vars.leveltext = ("%s (%s)"):format( vars.leveltext, _G["DUNGEON_DIFFICULTY" .. difficulty] );
        end--if
    end--if

    self.vars.zonetext = self:GetFactionColor( vars.current_zone, profile.showColoredText ) .. vars.zonetext;
end--Broker_RecZone:OnDataUpdate()

function Broker_RecZone:Update()
    self:OnDataUpdate();
    self:OnTextUpdate();
    self:UpdateTooltip();
end--Broker_RecZone:Update()

function Broker_RecZone:OnTextUpdate()
    local vars = self.vars;

    -- do I want to make one of these a value instead?
    self.ldbobj.text = vars.zonetext .. vars.coordtext .. ( self.db.profile.showLevelRange and vars.leveltext or "" );
end--Broker_RecZone:OnTextUpdate()

function Broker_RecZone:UpdateTooltip()
    if( self.vars.qtip ) then
        self:OnTooltipUpdate();
    end--if
end--Broker_RecZone:UpdateTooltip()

function Broker_RecZone:OnTooltipUpdate()
    local c, i;
    local profile = self.db.profile;
    local lower = profile.lower;
    local higher = profile.higher;
    local colorTooltip = profile.showColoredText;
    local vars = self.vars;
    local tooltip = vars.qtip;
    local line, column;
    local current_zone = vars.current_zone;
    local zones = vars.zones;
    local playerFaction = vars.playerFaction;

    tooltip:Clear();
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, L["Recommended Zone"], tooltip:GetHeaderFont(), nil, tooltip:GetColumnCount() );
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, " ", nil, nil, tooltip:GetColumnCount() );
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, L["Current Zone:"], nil, nil, -1 );
    line, column = tooltip:SetCell( line, column, current_zone.type < BRECZONE_DUNGEON and self:GetZoneText( current_zone, false ) or self:GetInstanceText( current_zone, false, GetInstanceDifficulty() ) );
    if( vars.subZoneName ~= "" ) then
        line, column = tooltip:AddLine();    
        line, column = tooltip:SetCell( line, column, L["Current Sub Zone:"], nil, nil, -1 );
        line, column = tooltip:SetCell( line, column, vars.subZoneName );
    end--if    
    if( current_zone.type < BRECZONE_COLLECTOR ) then
        -- Show Coords
        if( profile.showCoords ) then
            line, column = tooltip:AddLine();
            line, column = tooltip:SetCell( line, column, L["Coords:"], nil, nil, -1 );
            line, column = tooltip:SetCell( line, column, ("%.0f, %.0f"):format( vars.playerCoordX * 100, vars.playerCoordY * 100 ) );
        end--if
        -- Show faction
        if( profile.showFaction ) then
            line, column = tooltip:AddLine();
            line, column = tooltip:SetCell( line, column, L["Faction:"], nil, nil, -1 );
            line, column = tooltip:SetCell( line, column, self:GetFactionText( current_zone, colorTooltip ) );
        end--if
        -- show continent
        if( profile.showContinent ) then
            line, column = tooltip:AddLine();
            line, column = tooltip:SetCell( line, column, L["Continent:"], nil, nil, -1 );
            line, column = tooltip:SetCell( line, column, self:GetContinentText( current_zone, colorTooltip ) );
        end--if
    end--if

    -- Show border zones for current zone.
    if( profile.showCurInstance ) then
        if ( current_zone.zones ) then
            for _, i in ipairs( current_zone.zones ) do
                local f = zones[i].faction;
                local t = zones[i].type;

                if( ( ( t == BRECZONE_ZONE ) or ( t == BRECZONE_CITY ) ) and ( ( f == BRECZONE_CONTESTED ) or ( f == playerFaction ) or ( f == BRECZONE_SANCTUARY ) ) ) then
                    line, column = tooltip:AddLine();
                    line, column = tooltip:SetCell( line, column, L["Border Zone:"], nil, nil, -1 );
                    line, column = tooltip:SetCell( line, column, self:GetZoneText( zones[i], colorTooltip ) );
                end--if
            end--for
        end--if
    end --if

    -- Show instances for current zone.
    if( profile.showCurInstance ) then
        if( current_zone.zones ) then
            for _, i in ipairs( current_zone.zones ) do
                local f = zones[i].faction;
                local t = zones[i].type;

				if( not t ) then
					self:Print( "Error with zone " .. zones[i].name );                
                elseif( ( t > BRECZONE_COLLECTOR ) and ( ( f == BRECZONE_CONTESTED ) or ( f == playerFaction ) ) ) then
                    line, column = tooltip:AddLine();
                    line, column = tooltip:SetCell( line, column, L["Instance:"], nil, nil, -1 );
                    line, column = tooltip:SetCell( line, column, self:GetInstanceText( zones[i], colorTooltip ) );
                elseif( t == BRECZONE_COLLECTOR ) then
                    for _, j in ipairs( zones[i].zones ) do
                        local f = zones[j].faction;
                        local t = zones[j].type;

						if( not t ) then
							self:Print( "Error with zone " .. zones[j].name );                        
                        elseif( ( t > BRECZONE_COLLECTOR ) and ( ( f == BRECZONE_CONTESTED ) or ( f == playerFaction ) ) ) then
                            line, column = tooltip:AddLine();
                            line, column = tooltip:SetCell( line, column, L["Instance:"], nil, nil, -1 );
                            line, column = tooltip:SetCell( line, column, self:GetInstanceText( zones[j], colorTooltip ) );
                        end--if
                    end--if
                end--if
            end--for
        end--if
    end--for
    
    -- Show recommended zones.
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, " ", nil, nil, tooltip:GetColumnCount() );
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, L["Recommended Zones:"], tooltip:GetHeaderFont(), nil, tooltip:GetColumnCount() );

    for _,zone in ipairs( zones ) do
        if( ( zone.type == BRECZONE_ZONE ) and
                ( ( zone.low - higher ) <= vars.playerLevel ) and
                ( ( zone.high + lower ) >= vars.playerLevel ) and
                ( ( zone.faction == BRECZONE_CONTESTED ) or ( zone.faction == playerFaction ) ) ) then
            -- Show faction
            local factionText;

            if( profile.showFaction ) then
                factionText = self:GetFactionText( zone, colorTooltip );
            else
                factionText = "";
            end--if

            -- show continent
            local continentText;

            if( profile.showContinent ) then
                continentText = self:GetContinentText( zone, colorTooltip );
            else
                continentText = "";
            end--if

            line, column = tooltip:AddLine();
            line, column = tooltip:SetCell( line, column, self:GetZoneText( zone, colorTooltip ), nil, nil, nil, clickProvider, self, zone );
            line, column = tooltip:SetCell( line, column, factionText, nil, nil, nil, clickProvider, self, zone );
            line, column = tooltip:SetCell( line, column, continentText, nil, nil, nil, clickProvider, self, zone );
        end--if
    end--for

    -- Show recommended instances/battlegrounds/raids
    if( profile.showInstance or profile.showRaid or profile.showBattleground ) then
        line, column = tooltip:AddLine();
        line, column = tooltip:SetCell( line, column, " ", nil, nil, tooltip:GetColumnCount() );    
        line, column = tooltip:AddLine();
        line, column = tooltip:SetCell( line, column, L["Recommended Instances:"], tooltip:GetHeaderFont(), nil, tooltip:GetColumnCount() );
        for _, instance in ipairs( zones ) do
            local type = instance.type;

            if( ( not instance.NYI ) and ( type ~= BRECZONE_COLLECTOR ) and ( type ~= BRECZONE_ARENA ) and ( instance.low - higher <= self.vars.playerLevel ) and
                    ( instance.high + lower >= self.vars.playerLevel ) and
                    ( ( instance.faction == BRECZONE_CONTESTED ) or ( instance.faction == vars.playerFaction ) ) and
                    ( ( ( type == BRECZONE_DUNGEON ) and profile.showInstance ) or
                    ( ( type == BRECZONE_RAID ) and profile.showRaid ) or
                    ( ( ( type == BRECZONE_BATTLEGROUND ) or ( type == BRECZONE_PVP ) ) and profile.showBattleground ) ) ) then
                local text =  self:GetInstanceText( instance, colorTooltip );
                local locText = "";

                if( profile.showLoc and instance.zones ) then
                    local zone = zones[instance.zones[1]];

                    if( zone ) then
                        locText = "<" .. ( zone.displayname or zone.name ) .. ">";
                    end--if
                end--if
                line, column = tooltip:AddLine();
                line, column = tooltip:SetCell( line, column, text, nil, nil, -1, clickProvider, self, instance );
                line, column = tooltip:SetCell( line, column, locText, nil, nil, nil, clickProvider, self, instance );
            end--if
        end--for
    end--if
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, " ", nil, nil, tooltip:GetColumnCount() );
    line, column = tooltip:AddLine();
    line, column = tooltip:SetCell( line, column, L["Hint"], nil, nil, tooltip:GetColumnCount() );
end--Broker_RecZone:OnTooltipUpdate()                                                

function Broker_RecZone:OnClick( btn )
    if( IsShiftKeyDown() ) then
        if( ChatFrameEditBox:IsVisible() ) then
            local text = "";
            local vars = self.vars;
            local profile = self.db.profile;

            if( profile.showZoneName ) then
                text = vars.zoneName .. " ";
                if( profile.showSubZoneName ) then
                    if ( vars.subZoneName ~= "" ) then
                        text = vars.zoneName .. ": " .. vars.subZoneName .. " ";
                    end--if
                end--if
            elseif( profile.showSubZoneName ) then
                if( vars.subZoneName ~= "" ) then
                    text = vars.subZoneName .. " ";
                else
                    text = vars.zoneName .. " ";
                end--if
            end--if

            local x, y = GetPlayerMapPosition( "player" );

            text = text .. ("%.0f, %.0f"):format( x * 100, y * 100 );
            ChatFrameEditBox:Insert( text );
        end--if
    elseif( btn == RightButton ) then
        InterfaceOptionsFrame_OpenToCategory( L["Recommended Zone"] );
    else
        if( self.db.profile.showMap ) then
            Broker_RecZoneMapFrame:Show();
        else
            Broker_RecZoneMapFrame:Hide();
        end--if
    end--if
end--Broker_RecZone:OnClick()

function Broker_RecZone:OnClickZone( zone )
    local start = self:GetZoneNr( self.vars.current_zone );
    local goal = self:GetZoneNr( zone )
    local text = "";
    local tooltip =  self:CreateTooltip();

    self.vars.path = self:FindPath( start, goal );
    if( not self.vars.path ) then
        self.vars.path = self:FindPath( start, goal, true ); -- Try walking in opposing factions zones
    end--if
    if( not self.vars.path ) then
        tooltip:AddLine( "No path to destination found!" );
    else
        local zones = self.vars.zones;
        local showColoredText = self.db.profile.showColoredText;
        local pathSize = #self.vars.path;

        tooltip:AddLine( "Walk path" );
        for n, i in ipairs( self.vars.path ) do
            local text = self:GetPathText( zones[i], showColoredText );

            if( n ~= pathSize ) then
                tooltip:AddLine( text .. " ->" );
            else
                tooltip:AddLine( text );
            end--if
        end--for
    end--if
    self.vars.tooltip:Show();
end--Broker_RecZone:OnClickZone( zone )

function Broker_RecZone:CreateTooltip()
    local tooltip = self.vars.tooltip;
    local qtip = self.vars.qtip;

    if( not tooltip ) then
        tooltip = CreateFrame( "GameTooltip", "Broker_RecZoneTooltip", qtip, "GameTooltipTemplate" );
    end--if
    tooltip:SetOwner( qtip, "ANCHOR_NONE" );
    tooltip:ClearAllPoints();
    if( qtip:GetLeft() < GetScreenWidth() / 2 ) then
        tooltip:SetPoint( "LEFT", qtip, "RIGHT" );
    else
        tooltip:SetPoint( "RIGHT", qtip, "LEFT" );
    end--if
    self.vars.tooltip = tooltip;
    return tooltip;
end--Broker_RecZone:CreateTooltip()

function Broker_RecZone:FindPath( start, goal, opposing )
    local visited = {};
    local path = { start };
    local paths = { path };
    local zones = self.vars.zones;
    local function copytable( tbl, newitem )
        local newtbl = {};

        for _, a in ipairs( tbl ) do
            table.insert( newtbl, a );
        end--for
        table.insert( newtbl,newitem );
        return newtbl;
    end--function copytable( tbl,newitem )

    visited[start] = true;
    while true do
        path = paths[1];

        local c = path[table.getn( path )];

        if( zones[c].zones ) then
            for _, d in ipairs( zones[c].zones ) do
                if not visited[d] then
                    local z = zones[d];

                    if( ( z.type == BRECZONE_ZONE ) or ( z.faction == BRECZONE_CONTESTED ) or ( z.faction == self.vars.playerFaction ) or opposing  ) then
                        visited[d]=true;

                        local newpath = copytable( path, d );

                        if( d == goal ) then
                            paths = nil;
                            visited = nil;
                            return newpath; -- Path found
                        end--if
                        table.insert( paths, newpath );
                    end--if
                end--if
            end--for
        end--if
        table.remove( paths, 1 );
        if( table.getn( paths ) == 0 ) then
            return; -- Nothing found
        end--if
    end--while
end--Broker_RecZone:FindPath( start, goal, opposing )

function Broker_RecZone:GetZoneNr( zone )
    local name = zone.name;
    local faction = zone.faction;

    for i, z in ipairs( self.vars.zones ) do
        if( z.name:find( name, 1, true ) and ( z.faction == faction ) ) then
            return i;
        end--if
    end--for
end--Broker_RecZone:GetZoneNr( zone )


-- Worldmap function --

function Broker_RecZone:ToggleWorldMapText()
    if( self.db.profile.showMap ) then
        Broker_RecZoneMapFrame:Show();
    else
        Broker_RecZoneMapFrame:Hide();
    end--if
end--Broker_RecZone:ToggleWorldMapText()

function Broker_RecZone.MapFrame_OnUpdate()
    local self = Broker_RecZone;
    local zoneText = "";

    if( self.db.profile.showMapCoords ) then
        local OFFSET_X = 0.0022;
        local OFFSET_Y = -0.0262;
        local x, y = GetCursorPosition();

        x = x / WorldMapFrame:GetScale();
        y = y / WorldMapFrame:GetScale();

        local px, py = GetPlayerMapPosition( "player" );
        local centerX, centerY = WorldMapFrame:GetCenter();
        local width = WorldMapButton:GetWidth();
        local height = WorldMapButton:GetHeight();
        local adjustedX = ( x - ( centerX - ( width / 2 ) ) ) / width;
        local adjustedY = ( centerY + ( height / 2 ) - y ) / height;
        local cx = adjustedX + OFFSET_X;
        local cy = adjustedY + OFFSET_Y;
        
        Broker_RecZoneMapFrameCursorCoords:SetText( ("%s %.0f, %.0f"):format( L["Cursor:"], 100 * cx, 100 * cy ) );
        Broker_RecZoneMapFramePlayerCoords:SetText( ("%s %.0f, %.0f"):format( L["Player:"], 100 * px, 100 * py ) );
    end--if

    if( self.db.profile.showMap ) then
        local areaName = WorldMapFrame.areaName;

        if( areaName ~= nil ) then
            local zones = self.vars.zones;
            local playerFaction = self.vars.playerFaction;

            for i, zone in ipairs( zones ) do
                if( areaName:find( zone.name, 1, true ) ) then
                    if( zone.type == BRECZONE_PVP ) then
                        zoneText = ("%s(%d-%d %s)|r\n\n"):format( self:GetZoneColor( zone ), zone.low, zone.high, L["PVP"] );
                    else
                        zoneText = ("%s(%d-%d)|r\n\n"):format( self:GetZoneColor( zone ), zone.low, zone.high );
                    end--if
                    if( zone.zones ) then
                        local foundfirst = false;

                        for _, c in ipairs( zone.zones ) do
                            local cZones = zones[c];

							if( not cZones.type ) then
							    self:Print( "Error with zone " .. cZones.name );
                            elseif( cZones.type > BRECZONE_COLLECTOR ) then
                                if( not foundfirst ) then
                                    zoneText = zoneText .. HEX_YELLOW .. L["Instances:"] .. "|r\n";
                                    foundfirst = true;
                                end--if
                                zoneText = zoneText .. self:GetInstanceText( cZones, true ) .. ( cZones.NYI and ( " " .. L["[Future]"] ) or "" ) .. "\n";
                            elseif( cZones.type == BRECZONE_COLLECTOR ) then
                                for _, t in ipairs( cZones.zones ) do
                                    local tZones = zones[t];
        
                                    if( ( tZones.type > BRECZONE_COLLECTOR ) and
                                            ( ( tZones.faction == BRECZONE_CONTESTED ) or ( tZones.faction == playerFaction ) ) ) then
                                        if( not foundfirst ) then
                                            zoneText = zoneText .. HEX_YELLOW .. L["Instances:"] .. "|r\n";
                                            foundfirst = true;
                                        end--if
                                        zoneText = zoneText .. self:GetInstanceText( tZones, true ) .. ( tZones.NYI and ( " " .. L["[Future]"] ) or "" ) .. "\n";
                                    end--if
                                end--for
                            end--if
                        end--for
                    end--if
                end--if
            end--for
        end--if
        Broker_RecZoneMapText:SetText( zoneText );
    end--if
end--Broker_RecZone.MapFrame_OnUpdate()
